
# q-expansion of newform 483.2.a.i, downloaded from the LMFDB on 26 December 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 483
weight = 2
poly_data = [2, -1, -6, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[-3, 0, 1, 0], 1], [[-1, -5, 0, 1], 1]]

hecke_ring_character_values = None
aps_data = [[0, -1, 0, 0], [-1, 0, 0, 0], [1, 0, 0, -1], [1, 0, 0, 0], [-1, 0, -1, 1], [2, -1, 1, 1], [3, 1, -1, 0], [1, -2, -1, 1], [-1, 0, 0, 0], [1, 1, 1, -2], [1, 1, -1, 0], [5, 3, -1, 0], [1, 0, 1, 1], [2, 3, 1, -1], [2, -1, 0, 1], [-1, 1, -2, 2], [-1, 7, 0, -2], [6, -2, -1, 0], [1, 2, 0, 3], [-4, -1, 1, 1], [5, -1, 1, 4], [-3, -1, 1, -2], [1, 1, 3, -4], [0, -1, 1, 3], [-1, 3, -1, -2], [-3, 3, 2, 2], [2, 3, -6, 1], [-8, 1, -1, -1], [5, -6, -2, 1], [-7, 0, 2, -1], [0, -10, 1, 4], [7, -2, 3, -1], [7, 0, 3, -5], [9, -4, 2, 1], [-2, 5, -2, 1], [-6, -5, 4, -3], [4, -3, 0, 3], [3, -1, 6, -2], [1, 0, 1, 3], [9, -3, 1, 2], [-3, -8, 2, 1], [-1, -1, 3, -2], [-6, 3, 0, 5], [0, -3, 4, -5], [4, -3, -1, 5], [-1, -3, 0, 0], [5, 2, -1, 9], [-3, 10, 2, -1], [-3, -2, 2, -9], [5, 11, 0, -2], [-3, 4, 0, 3], [-23, 2, 0, -1], [-1, -2, 3, 1], [-10, -6, 0, -4], [2, -7, -2, -3], [-1, -2, -7, -1], [0, 3, -3, 3], [1, -3, 1, -8], [14, 2, -3, 0], [-10, -2, -6, 6], [-13, 2, -4, -3], [4, -8, 2, 2], [3, -7, -3, 2], [-6, 0, 9, 0], [14, 5, 0, 1], [7, -2, 4, 3], [-4, 3, -4, -5], [-2, -3, -1, -5], [3, -1, -7, 0], [-1, 4, 2, 5], [7, 3, 3, 6], [-3, 18, 0, -5], [-4, 14, 1, -8], [-3, -5, 7, 0], [0, 2, 4, -2], [-13, -1, -3, 4], [9, -9, -1, 8], [-8, -8, 0, -6], [-11, -2, -1, -1], [-9, -7, 5, 0], [3, 10, 2, 1], [0, 7, -1, -5], [-14, -14, 7, -2], [-4, 3, 4, -11], [-7, 11, 3, 2], [0, -8, 0, -4], [-5, 6, 4, -5], [-5, 0, -6, 1], [3, -6, -8, 3], [-3, 4, 9, 3], [-1, 3, 5, 0], [3, -1, -1, 0], [-3, -7, -3, -8], [-10, -9, 7, -5], [15, 4, -6, 7], [-7, -4, -2, 5], [-10, 1, -2, -3], [-14, 4, -4, -4], [20, 7, 4, -9], [6, 1, -6, 5], [7, -4, -2, -1], [-10, 2, 6, -2], [10, -3, -5, 5], [2, 7, 4, 7], [-21, -1, -1, -4], [-6, 12, -4, -4], [8, 4, -13, 2], [14, -6, -2, -6], [2, 15, -5, -5], [-25, 8, 0, -7], [14, -3, -1, 13], [11, 7, -5, 10], [-10, -3, -5, 3], [-6, -3, 7, -11], [19, 3, 9, -8], [-18, 0, 3, 6], [16, 0, 1, -6], [7, -2, 4, -7], [9, 4, 4, 3], [-27, 1, -3, -4], [-11, -14, 1, 3], [-9, -8, -5, 11], [10, -15, -3, 3], [13, 3, -1, 10], [18, -5, 1, 7], [5, 9, 10, -8], [18, -3, -1, -5], [-13, -11, -7, 10], [31, -10, -3, -1], [-2, 22, 2, -6], [-9, -7, 5, -2], [-1, 13, -6, -4], [9, 12, -6, 5], [-1, -1, -11, 2], [14, -11, 4, 1], [2, -6, -2, 6], [11, -1, -7, -2], [7, -13, 4, 10], [-12, 4, 8, -6], [-2, 5, 15, -5], [-5, 1, -9, 6], [20, -10, -8, -4], [3, 3, 0, 10], [9, 3, 0, 4], [-23, -5, 3, -4], [-12, -7, -7, 3], [-2, -8, 6, 4], [-8, -1, -4, -3], [10, 18, -6, -4], [6, -2, 8, -4], [-26, 11, -4, -5], [-2, -10, -2, -2], [-4, 9, 3, 11], [-20, 5, 3, -5], [-7, 4, 6, -7], [-20, -2, -6, -10], [11, -13, -7, 0], [-1, -12, 10, 1], [-13, 0, -3, 11], [-8, 0, 2, 6], [2, 6, 2, 8], [8, -12, -7, 8], [10, 6, -6, -4], [26, 1, 1, 5], [14, -2, 7, 12], [19, -1, 5, -4], [-8, 8, -5, 10], [1, 21, -5, -10]]
